@@ -49,6 +49,8 @@ static GUPnPDIDLLiteParser *didl_parser;
 
 typedef struct
 {
+  GUPnPServiceProxy *content_dir;
+
   gchar *id;
 
   guint32 starting_index;
@@ -64,12 +66,14 @@ typedef struct
 } BrowseMetadataData;
 
 static BrowseData *
-browse_data_new (const char *id,
-                 guint32     starting_index)
+browse_data_new (GUPnPServiceProxy *content_dir,
+                 const char        *id,
+                 guint32            starting_index)
 {
         BrowseData *data;
 
         data = g_slice_new (BrowseData);
+        data->content_dir = g_object_ref (content_dir);
         data->id = g_strdup (id);
         data->starting_index = starting_index;
 
@@ -80,6 +84,7 @@ static void
 browse_data_free (BrowseData *data)
 {
         g_free (data->id);
+        g_object_unref (data->content_dir);
         g_slice_free (BrowseData, data);
 }
 
@@ -545,10 +550,10 @@ on_container_update_ids (GUPnPServiceProxy *content_dir,
 }
 
 static void
-append_didle_object (xmlNode           *object_node,
-                     GUPnPServiceProxy *content_dir,
-                     GtkTreeModel      *model,
-                     GtkTreeIter       *server_iter)
+append_didle_object (xmlNode      *object_node,
+                     BrowseData   *browse_data,
+                     GtkTreeModel *model,
+                     GtkTreeIter  *server_iter)
 {
         GtkTreeIter parent_iter;
         char       *id;
@@ -586,7 +591,8 @@ append_didle_object (xmlNode           *object_node,
                 icon = get_item_icon (object_node);
         }
 
-        if (strcmp (parent_id, "0") == 0) {
+        /* Check if we browsed the root container. */
+        if (strcmp (browse_data->id, "0") == 0) {
                 parent_iter = *server_iter;
         } else if (!find_row (model,
                               server_iter,
@@ -601,7 +607,7 @@ append_didle_object (xmlNode           *object_node,
                                            NULL, &parent_iter, position,
                                            0, icon,
                                            1, title,
-                                           3, content_dir,
+                                           3, browse_data->content_dir,
                                            4, id,
                                            5, is_container,
                                            -1);
@@ -658,17 +664,17 @@ on_didl_object_available (GUPnPDIDLLiteParser *didl_parser,
                           xmlNode             *object_node,
                           gpointer             user_data)
 {
-        GUPnPServiceInfo *content_dir;
-        GtkTreeModel     *model;
-        GtkTreeIter       server_iter;
-        const char       *udn;
+        BrowseData   *browse_data;
+        GtkTreeModel *model;
+        GtkTreeIter   server_iter;
+        const char   *udn;
 
         model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
         g_assert (model != NULL);
 
-        content_dir = GUPNP_SERVICE_INFO (user_data);
+        browse_data = (BrowseData *) user_data;
 
-        udn = gupnp_service_info_get_udn (content_dir);
+        udn = gupnp_service_info_get_udn (browse_data->content_dir);
 
         if (find_row (model,
                        NULL,
@@ -677,7 +683,7 @@ on_didl_object_available (GUPnPDIDLLiteParser *didl_parser,
                        (gpointer) udn,
                        FALSE)) {
                 append_didle_object (object_node,
-                                     GUPNP_SERVICE_PROXY (content_dir),
+                                     browse_data,
                                      model,
                                      &server_iter);
         }
@@ -723,7 +729,7 @@ browse_cb (GUPnPServiceProxy       *content_dir,
                                                 (didl_parser,
                                                  didl_xml,
                                                  on_didl_object_available,
-                                                 content_dir,
+                                                 data,
                                                  &error)) {
                         g_warning ("%s\n", error->message);
                         g_error_free (error);
@@ -753,7 +759,6 @@ browse_cb (GUPnPServiceProxy       *content_dir,
         }
 
         browse_data_free (data);
-        g_object_unref (content_dir);
 }
 
 static void
@@ -764,9 +769,9 @@ browse (GUPnPServiceProxy *content_dir,
 {
         BrowseData *data;
 
-        g_object_ref (content_dir);
-
-        data = browse_data_new (container_id, starting_index);
+        data = browse_data_new (content_dir,
+                                container_id,
+                                starting_index);
 
         gupnp_service_proxy_begin_action
                                 (content_dir,
